import React from "react";
import Taro from "@tarojs/taro";
import { View } from "@tarojs/components";
import { PowerScrollView } from "@antmjs/vantui";
import useShow from "@/layouts/hooks/useShow";

export default React.forwardRef((props, ref) => {
  const { request, params, renderItem, onLoadingChange, onDataChange } = props;

  const pageSize = 5;
  const pageNumRef = React.useRef(0);
  const [loading, setLoading] = React.useState(false);
  const [list, setList] = React.useState([]);
  const [finished, setFinished] = React.useState(true);
  const listRef = React.useRef([]);
  const paramsRef = React.useRef();
  listRef.current = list;
  paramsRef.current = params || {};

  const changeLoading = (val) => {
    setLoading(val);
    if (onLoadingChange) {
      onLoadingChange(val);
    }
  };

  const queryData = React.useCallback(
    (options = {}) => {
      return new Promise((resolve, reject) => {
        changeLoading(true);
        request({
          pageSize,
          ...paramsRef.current,
          ...options,
        })
          .then((res) => {
            const { records, current, pages } = res;

            let dataset =
              current == 1
                ? records || []
                : listRef.current.concat(records || []);
            if (onDataChange) {
              dataset = onDataChange(dataset);
            }
            setList(dataset);

            setFinished(current >= pages);
            changeLoading(false);
            resolve();
          })
          .catch((err) => {
            changeLoading(false);
            reject(err);
          });
      });
    },
    []
  );

  const refresh = React.useCallback(() => {
    console.log(pageNumRef);
    pageNumRef.current = 1;

    queryData({ pageNum: pageNumRef.current });
    console.log("refresh" + pageNumRef.current);
  }, [queryData]);

  const onScrollToLower = React.useCallback(
    (event = 0, isRefresh = false) => {
      console.log("onScrollToLower" + pageNumRef.current);
      pageNumRef.current += 1;
      queryData({ pageNum: pageNumRef.current });
    },
    []
  );

  useShow(() => {
    refresh();
  });

  
  React.useEffect(() => {
    refresh();
  }, [refresh, params]);

  React.useImperativeHandle(ref, () => ({
    refresh,
    updateData: (dt) => setList(dt),
  }));
 

  return (
    <PowerScrollView
      scrollY
      finishedText="没有更多了"
      refresherEnabled={false}
      current={list.length}
      pageSize={pageSize}
      finished={finished}
      onScrollToLower={onScrollToLower}
      style={{ height: "100%", overflowY: "auto" }}
      renderLoading={loading ? <view>加载中...</view> : <view></view>}
    >
      {list.map(renderItem)}
    </PowerScrollView>
  );
});
